@@ -14,8 +14,8 @@ def _get_sanitizer(name="default"):
     elif name == "default":
         return Sanitizer()
     raise ImproperlyConfigured(
-        "Unknown sanitizer %r, did you define HTML_SANITIZERS[%r] in your"
-        " Django settings module?" % (name, name)
+        "Unknown sanitizer {!r}, did you define HTML_SANITIZERS[{!r}] in your"
+        " Django settings module?".format(name, name)
     )
 
 
@@ -32,7 +32,7 @@ def check_configuration(app_configs, **kwargs):
         except TypeError as exc:
             errors.append(
                 checks.Error(
-                    "Invalid sanitizer configuration '%s': %s" % (name, exc),
+                    f"Invalid sanitizer configuration '{name}': {exc}",
                     id="html_sanitizer.E001",
                 )
             )
