@@ -45,17 +45,26 @@ typographic_whitespace = "".join(
 
 
 def normalize_overall_whitespace(
-    html, keep_typographic_whitespace=False, whitespace_re=None
+    html, *, keep_typographic_whitespace=False, whitespace_re=None
 ):
-    # remove all sorts of newline and nbsp characters
-    whitespace = ["\n", "&#10;", "&#xa;", "\r", "&#13;", "&#xd;"]
-    if not keep_typographic_whitespace:
-        # non-breaking space representations
-        whitespace += ["\xa0", "&nbsp;", "&#160;", "&#xa0;"]
-    if whitespace_re is None:
-        whitespace_re = r"\s+"
+    if keep_typographic_whitespace:
+        return html
+    whitespace = [
+        "\xa0",
+        "&nbsp;",
+        "&#160;",
+        "&#xa0;",
+        "\n",
+        "&#10;",
+        "&#xa;",
+        "\r",
+        "&#13;",
+        "&#xd;",
+    ]
     for ch in whitespace:
         html = html.replace(ch, " ")
+    if whitespace_re is None:
+        whitespace_re = r"\s+"
     html = re.sub(whitespace_re, " ", html)
     return html
 
@@ -103,17 +112,19 @@ def anchor_id_to_name(element):
     return element
 
 
-def normalize_whitespace_in_text_or_tail(element, whitespace_re=None):
+def normalize_whitespace_in_text_or_tail(
+    element, *, whitespace_re=None, keep_typographic_whitespace=False
+):
     if whitespace_re is None:
         whitespace_re = re.compile(r"\s+")
-    if element.text:
+    if element.text and not keep_typographic_whitespace:
         while True:
             text = whitespace_re.sub(" ", element.text)
             if element.text == text:
                 break
             element.text = text
 
-    if element.tail:
+    if element.tail and not keep_typographic_whitespace:
         while True:
             text = whitespace_re.sub(" ", element.tail)
             if element.tail == text:
@@ -164,33 +175,36 @@ DEFAULT_SETTINGS = {
 }
 
 
-class Sanitizer(object):
+def coerce_to_set(value):
+    if isinstance(value, set):
+        return value
+    elif isinstance(value, (tuple, list)):
+        return set(value)
+    raise TypeError(f"Expected a set but got value {value!r} of type {type(value)}")
+
+
+class Sanitizer:
     def __init__(self, settings=None):
         self.__dict__.update(DEFAULT_SETTINGS)
         self.__dict__.update(settings or {})
 
         # Allow iterables of any kind, not just sets.
-        self.tags = set(self.tags)
-        self.empty = set(self.empty)
-        self.separate = set(self.separate)
-        self.whitespace = set(self.whitespace)
+        self.tags = coerce_to_set(self.tags)
+        self.empty = coerce_to_set(self.empty)
+        self.separate = coerce_to_set(self.separate)
+        self.whitespace = coerce_to_set(self.whitespace)
+        self.attributes = {
+            tag: coerce_to_set(attributes)
+            for tag, attributes in self.attributes.items()
+        }
 
         if self.keep_typographic_whitespace:
             re_whitespace = r"[^\S%s]" % typographic_whitespace
         else:
             re_whitespace = r"\s"
-        import sys
 
-        if int(sys.version.split(".")[0]) == 2:
-            force_unicode_prefix = "(?u)"
-        else:
-            force_unicode_prefix = ""
-        self.only_whitespace_re = re.compile(
-            r"%s^%s*$" % (force_unicode_prefix, re_whitespace)
-        )
-        self.whitespace_re = re.compile(
-            r"%s%s+" % (force_unicode_prefix, re_whitespace)
-        )
+        self.only_whitespace_re = re.compile(rf"^{re_whitespace}*$")
+        self.whitespace_re = re.compile(rf"{re_whitespace}+")
 
         # Validate the settings.
         if not self.tags:
@@ -202,16 +216,17 @@ class Sanitizer(object):
             )
         if not self.tags.issuperset(self.empty):
             raise TypeError(
-                'Tags in "empty", but not allowed: %r' % (self.empty - self.tags,)
+                f'Tags in "empty", but not allowed: {self.empty - self.tags!r}'
             )
         if not self.tags.issuperset(self.separate):
             raise TypeError(
-                'Tags in "separate", but not allowed: %r' % (self.separate - self.tags,)
+                'Tags in "separate", but not allowed: {!r}'.format(
+                    self.separate - self.tags
+                )
             )
         if not self.tags.issuperset(self.attributes.keys()):
             raise TypeError(
-                'Tags in "attributes", but not allowed: %r'
-                % (set(self.attributes.keys()) - self.tags,)
+                f'Tags in "attributes", but not allowed: {set(self.attributes.keys()) - self.tags!r}'
             )
 
         anchor_attributes = self.attributes.get("a", ())
@@ -277,7 +292,9 @@ class Sanitizer(object):
                 element = processor(element)
 
             element = normalize_whitespace_in_text_or_tail(
-                element, whitespace_re=self.whitespace_re
+                element,
+                whitespace_re=self.whitespace_re,
+                keep_typographic_whitespace=self.keep_typographic_whitespace,
             )
 
             # remove empty tags if they are not explicitly allowed
@@ -347,12 +364,12 @@ class Sanitizer(object):
                     # tags of a mergeable type.
                     if nx.text:
                         if len(element):
-                            list(element)[-1].tail = "%s %s" % (
+                            list(element)[-1].tail = "{} {}".format(
                                 list(element)[-1].tail or "",
                                 nx.text,
                             )
                         else:
-                            element.text = "%s %s" % (element.text or "", nx.text)
+                            element.text = "{} {}".format(element.text or "", nx.text)
 
                     for child in nx:
                         element.append(child)
@@ -369,7 +386,7 @@ class Sanitizer(object):
 
             # remove all attributes which are not explicitly allowed
             allowed = self.attributes.get(element.tag, [])
-            for key in element.keys():
+            for key in element.keys():  # noqa: SIM118 (do not remove .keys())
                 if key not in allowed:
                     del element.attrib[key]
 
@@ -379,7 +396,9 @@ class Sanitizer(object):
                 element.set("href", self.sanitize_href(href))
 
             element = normalize_whitespace_in_text_or_tail(
-                element, whitespace_re=self.whitespace_re
+                element,
+                whitespace_re=self.whitespace_re,
+                keep_typographic_whitespace=self.keep_typographic_whitespace,
             )
 
         if self.autolink is True:
